#ifndef LAZY_MPL_BOOL_HPP_VK_20043010
#define LAZY_MPL_BOOL_HPP_VK_20043010

// (C) Copyright Vesa Karvonen 2004.
//
// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE.)

#include "lazy_mpl/quote.hpp"

// `BOOL<v>' represents a boolean.
template<bool v>
struct BOOL
  : QUOTE< BOOL<v> >
{
  static const bool value = v;
};

// `TRUE' and `FALSE' typedefs are provided for convenience.
typedef BOOL<true> TRUE;
typedef BOOL<false> FALSE;

// `NOT<b>' computes the logical negation of `b'.
template<class b>
struct NOT
  : BOOL<!b::type::value> {};

namespace detail {
  struct select_alternative {
    template<class c, class a>
    struct apply
      : QUOTE<a> {};
  };

  struct select_consequent {
    template<class c, class a>
    struct apply
      : QUOTE<c> {};
  };

  template<class b>
  struct select {};

  template<>
  struct select<FALSE>
    : QUOTE<select_alternative> {};

  template<>
  struct select<TRUE>
    : QUOTE<select_consequent> {};
}

// `SELECT<b>' evaluates to a binary metafunction class that returns
// the first parameter if `b' is `TRUE' and to the second parameter
// if `b' is `FALSE'.
template<class b>
struct SELECT
  : detail::select<typename b::type> {};

// `OR<l, r>' evaluates to `TRUE' if `l' is `TRUE' and to `r' if `b'
// is `FALSE'
template<class l, class r>
struct OR
  : detail::select<typename l::type>::type::template apply<TRUE, r>::type {};

// `AND<l, r>' evaluates to `FALSE' if `l' is `FALSE' and to `r' if
// `b' is `FALSE'
template<class l, class r>
struct AND
  : detail::select<typename l::type>::type::template apply<r, FALSE>::type {};

// `IF<l, r>' evaluates to `c' if `b' is TRUE and to `a' if `b' is
// `FALSE'.
template<class b, class c, class a>
struct IF
  : detail::select<typename b::type>::type::template apply<c, a>::type {};

#endif
