#ifndef LAZY_MPL_INT_HPP_VK_20043010
#define LAZY_MPL_INT_HPP_VK_20043010

// (C) Copyright Vesa Karvonen 2004.
//
// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE.)

#include "lazy_mpl/bool.hpp"

// `INT<v>' represents an integer.
//
// NOTE: The value of `INT<v>' is actually a `long'.
template<long v>
struct INT
  : QUOTE< INT<v> >
{
  static const long value = v;
};

// `ZERO' and `ONE' typedefs are provided for convenience.
typedef INT<0> ZERO;
typedef INT<1> ONE;

// `IS_0<i>' evaluates to `TRUE' if the value of `i' is 0 and
// otherwise to `FALSE'.
template<class i>
struct IS_0
  :  BOOL<(0 == i::type::value)> {};

// `ISNT_0<i>' evaluates to `TRUE' if the value of `i' isn't 0 and
// otherwise to `FALSE'.
template<class i>
struct ISNT_0
  :  BOOL<(0 != i::type::value)> {};

#define LAZY_MPL_INT_REL_OP(NAME, op)                   \
template<class l, class r>                              \
struct NAME                                             \
  : BOOL<(l::type::value op r::type::value)> {}

LAZY_MPL_INT_REL_OP(EQ,  ==);
LAZY_MPL_INT_REL_OP(NEQ, !=);
LAZY_MPL_INT_REL_OP(LT,  < );
LAZY_MPL_INT_REL_OP(LEQ, <=);
LAZY_MPL_INT_REL_OP(GT,  > );
LAZY_MPL_INT_REL_OP(GEQ, >=);
#undef LAZY_MPL_INT_REL_OP

#define LAZY_MPL_INT_UOP(NAME, var, exp)        \
template<class var>                             \
struct NAME                                     \
  : INT<exp> {}

LAZY_MPL_INT_UOP(NEG,  x, (-x::type::value));
LAZY_MPL_INT_UOP(SUCC, x, (x::type::value + 1));
LAZY_MPL_INT_UOP(PRED, x, (x::type::value - 1));
#undef LAZY_MPL_INT_UOP

#define LAZY_MPL_INT_BOP(NAME, op)                      \
template<class l, class r>                              \
struct NAME                                             \
  : INT<(l::type::value op r::type::value)> {}

LAZY_MPL_INT_BOP(PLUS,      +);
LAZY_MPL_INT_BOP(MINUS,     -);
LAZY_MPL_INT_BOP(TIMES,     *);
LAZY_MPL_INT_BOP(QUOTIENT,  /);
LAZY_MPL_INT_BOP(REMAINDER, %);
#undef LAZY_MPL_INT_BOP

#endif
