#ifndef LAZY_MPL_LIST_HPP_VK_20043010
#define LAZY_MPL_LIST_HPP_VK_20043010

// (C) Copyright Vesa Karvonen 2004.
//
// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE.)

#include "lazy_mpl/bool.hpp"

// `NIL' represents an empty list.
struct NIL
  : QUOTE< NIL > {};

namespace detail {
  template<class l>
  struct is_nil
    : FALSE {};

  template<>
  struct is_nil<NIL>
    : TRUE {};
}

// `IS_NIL<l>' evaluates to `TRUE' if `l' is `NIL' and otherwise to
// `FALSE'.
template<class l>
struct IS_NIL
  : detail::is_nil<typename l::type> {};

// `CONS<h, t>' is the list constructor. `CONS<h, t>' prepends
// the element `h' to the head of the list `t'. `CONS' is lazy.
// `CONS<h, t>' doesn't evaluate `h' or `t'.
//
// See also: `STRICT<l>'
template<class h, class t>
struct CONS
  : QUOTE< CONS<h, t> > {};

namespace detail {
  template<class l>
  struct is_cons
    : FALSE {};

  template<class h, class t>
  struct is_cons< CONS<h, t> >
    : TRUE {};
}

// `IS_CONS<l>' evaluates to `TRUE' if `l' is a `CONS' and otherwise
// to `FALSE'.
template<class l>
struct IS_CONS
  : detail::is_cons<typename l::type> {};

namespace detail {
  template<class t>
  struct head {};

  template<class h, class t>
  struct head< CONS<h, t> >
    : h {};
}

// `HEAD<l>' evaluates to the head of the list `l' assuming that `l'
// is a `CONS'.
template<class l>
struct HEAD
  : detail::head<typename l::type> {};

namespace detail {
  template<class _>
  struct tail {};

  template<class h, class t>
  struct tail< CONS<h, t> >
    : t {};
}

// `TAIL<l>' evaluates to the tail of the list `l' assuming that `l'
// is a `CONS'.
template<class l>
struct TAIL
  : detail::tail<typename l::type> {};

#endif
