// (C) Copyright Vesa Karvonen 2004.
//
// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE.)

#include "lazy_mpl/append.hpp"
#include "lazy_mpl/drop.hpp"
#include "lazy_mpl/filter.hpp"
#include "lazy_mpl/fold.hpp"
#include "lazy_mpl/int.hpp"
#include "lazy_mpl/iterate.hpp"
#include "lazy_mpl/map.hpp"
#include "lazy_mpl/op.hpp"
#include "lazy_mpl/strict.hpp"
#include "lazy_mpl/take.hpp"
#include "lazy_mpl/zip_with.hpp"
#include "test.hpp"

template<class r>
struct NOT_DIVISIBLE_BY : QUOTE<NOT_DIVISIBLE_BY<r> > {
  template<class l>
  struct apply
    : ISNT_0< REMAINDER<l, r> > {};
};

template<class l>
struct SIEVE
  : CONS<HEAD<l>,
         SIEVE<FILTER<NOT_DIVISIBLE_BY<HEAD<l> >,
                      TAIL<l> > > > {};

template<class l>
struct ADD : QUOTE<ADD<l> > {
  template<class r>
  struct apply
    : INT<(l::type::value + r::type::value)> {};
};

// `primes' is the list of all primes (less than LONG_MAX, of
// course).
struct primes
  : CONS<INT<2>,
         SIEVE<ITERATE<ADD<INT<2> >, INT<3> > > > {};

// `naturals' is the list of all naturals (less than LONG_MAX, of
// course).
struct naturals
  : ITERATE<OP_1<SUCC>, INT<0> > {};

// `fibs' is the list of all Fibonacci numbers (less than LONG_MAX,
// of course).
struct fibs
  : CONS<INT<0>,
         CONS<INT<1>,
              ZIP_WITH<OP_2<PLUS>, fibs, TAIL<fibs> > > > {};

typedef test< INT<0>,
              HEAD<naturals> > TEST;

typedef test< INT<20>,
              HEAD<DROP<INT<20>, naturals> > > TEST;

typedef test< INT<832040>,
              HEAD<DROP<INT<30>, fibs> > > TEST;

typedef test< INT<(9*10)/2>,
              FOLDR<OP_2<PLUS>, TAKE<INT<10>, naturals>, INT<0> > > TEST;

typedef test< INT<(9*10)/2>,
              FOLDL<OP_2<PLUS>, INT<0>, TAKE<INT<10>, naturals> > > TEST;

typedef test< CONS<INT<2>,
                   CONS<INT<3>,
                        CONS<INT<5>,
                             CONS<INT<7>,
                                  CONS<INT<11>, NIL> > > > >,
              STRICT< TAKE<INT<5>, primes> > > TEST;

int main() {}
